@@ -37,6 +37,11 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
             "pattern": r"https://media-files\.bunkr\.is/_-RnHoW69L\.mp4",
             "content": "80e61d1dbc5896ae7ef9a28734c747b28b320471",
         }),
+        # cdn4
+        ("https://bunkr.is/a/iXTTc1o2", {
+            "pattern": r"https://(cdn|media-files)4\.bunkr\.is/",
+            "content": "da29aae371b7adc8c5ef8e6991b66b69823791e8",
+        }),
         ("https://bunkr.to/a/Lktg9Keq"),
     )
 
@@ -63,22 +68,25 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
         url = self.root + "/a/" + self.album_id
 
         try:
-            data = json.loads(text.extract(
+            data = json.loads(text.extr(
                 self.request(url).text,
-                'id="__NEXT_DATA__" type="application/json">', '<')[0])
-            props = data["props"]["pageProps"]
-            album = props["album"]
-            files = props["files"]
+                'id="__NEXT_DATA__" type="application/json">', '<'))
+            album = data["props"]["pageProps"]["album"]
+            files = album["files"]
         except Exception as exc:
             self.log.debug(exc.__class__.__name__, exc)
             self.root = self.root.replace("bunkr", "app.bunkr", 1)
             return self._fetch_album_api(album_id)
 
+        headers = {"Referer": "https://stream.bunkr.is/"}
+
         for file in files:
             name = file["name"]
             cdn = file["cdn"]
-            if name.endswith(".mp4"):
-                cdn = cdn.replace("//cdn", "//media-files")
+            if name.endswith((".mp4", ".m4v", ".mov", ".webm",
+                              ".zip", ".rar", ".7z")):
+                cdn = cdn.replace("//cdn", "//media-files", 1)
+                file["_http_headers"] = headers
             file["file"] = cdn + "/" + name
 
         return files, {
