@@ -23,9 +23,9 @@ class KhinsiderSoundtrackExtractor(AsynchronousMixin, Extractor):
     root = "https://downloads.khinsider.com"
     test = (("https://downloads.khinsider.com"
              "/game-soundtracks/album/horizon-riders-wii"), {
-        "pattern": r"https?://vgm(site|downloads).com"
+        "pattern": r"https?://vgm(site|downloads)\.com"
                    r"/soundtracks/horizon-riders-wii/[^/]+"
-                   r"/Horizon%20Riders%20Wii%20-%20Full%20Soundtrack.mp3",
+                   r"/Horizon%20Riders%20Wii%20-%20Full%20Soundtrack\.mp3",
         "keyword": {
             "album": {
                 "count": 1,
@@ -76,15 +76,14 @@ class KhinsiderSoundtrackExtractor(AsynchronousMixin, Extractor):
             else:
                 fmt = fmt.lower().split(",")
 
-        page = text.extract(page, '<table id="songlist">', '</table>')[0]
+        page = text.extr(page, '<table id="songlist">', '</table>')
         for num, url in enumerate(text.extract_iter(
                 page, '<td class="clickable-row"><a href="', '"'), 1):
             url = text.urljoin(self.root, url)
             page = self.request(url, encoding="utf-8").text
             track = first = None
 
-            for url in text.extract_iter(
-                    page, 'style="color: #21363f;" href="', '"'):
+            for url in text.extract_iter(page, '<p><a href="', '"'):
                 track = text.nameext_from_url(url, {"num": num, "url": url})
                 if first is None:
                     first = track
