@@ -27,10 +27,6 @@ class PhilomenaExtractor(BooruExtractor):
     def _prepare(post):
         post["date"] = text.parse_datetime(post["created_at"])
 
-    @staticmethod
-    def _extended_tags(post):
-        pass
-
     def _pagination(self, url, params):
         params["page"] = 1
         params["per_page"] = self.per_page
@@ -122,7 +118,7 @@ class PhilomenaPostExtractor(PhilomenaExtractor):
                 "tag_ids": list,
                 "tags": list,
                 "thumbnails_generated": True,
-                "updated_at": "2022-04-25T09:30:57Z",
+                "updated_at": r"re:\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\dZ",
                 "uploader": "Clover the Clever",
                 "uploader_id": 211188,
                 "upvotes": int,
